@@ -1,8 +1,28 @@
+"""
+Copyright (c) 2022 Poutyne and all respective contributors.
+
+Each contributor holds copyright over their respective contributions. The project versioning (Git)
+records all such contribution source information.
+
+This file is part of Poutyne.
+
+Poutyne is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
+License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later
+version.
+
+Poutyne is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
+of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
+
+You should have received a copy of the GNU Lesser General Public License along with Poutyne. If not, see
+<https://www.gnu.org/licenses/>.
+"""
+
 import math
 from tempfile import TemporaryDirectory
 from unittest import TestCase, skipIf
 from unittest.mock import MagicMock, call, ANY
 
+
 import torch
 import torch.nn as nn
 
@@ -13,38 +33,10 @@ try:
 except ImportError:
     SummaryWriter = None
 
-from poutyne import Model, TensorBoardGradientTracker, WeightsGradientsStatsTracker, Tracker
-
-
-class TrackerTest(TestCase):
-
-    def test_keep_good_layer(self):
-        # pylint: disable=protected-access
-        tracker = Tracker(keep_bias=False)
-        layer_to_keep_params = MagicMock()
-        layer_to_keep_params.requires_grad = True
-        self.assertTrue(tracker._keep_layer(layer_to_keep_params, "fake_layer_name_to_keep"))
-        self.assertFalse(tracker._keep_layer(layer_to_keep_params, "bias_name_not_to_keep"))
-
-        layer_not_to_keep_params = MagicMock()
-        layer_not_to_keep_params.requires_grad = False
-        self.assertFalse(tracker._keep_layer(layer_not_to_keep_params, "fake_layer_name_not_to_keep"))
-        self.assertFalse(tracker._keep_layer(layer_not_to_keep_params, "bias_name_not_to_keep"))
-
-        tracker = Tracker(keep_bias=True)
-        layer_to_keep_params = MagicMock()
-        layer_to_keep_params.requires_grad = True
-        self.assertTrue(tracker._keep_layer(layer_to_keep_params, "fake_layer_name_to_keep"))
-        self.assertTrue(tracker._keep_layer(layer_to_keep_params, "bias_name_to_keep"))
-
-        layer_not_to_keep_params = MagicMock()
-        layer_not_to_keep_params.requires_grad = False
-        self.assertFalse(tracker._keep_layer(layer_not_to_keep_params, "fake_layer_name_not_to_keep"))
-        self.assertFalse(tracker._keep_layer(layer_not_to_keep_params, "bias_name_not_to_keep"))
+from poutyne import Model, TensorBoardGradientTracker, WeightsGradientsStatsTracker, GradientTracker
 
 
 class GradientStatsTrackerTest(TestCase):
-
     def setUp(self):
         self.tracker = WeightsGradientsStatsTracker(number_layers=2)
 
@@ -76,7 +68,7 @@ class GradientStatsTrackerTest(TestCase):
                 "min": self.layer_1_min,
                 "abs_min": self.absolute_min_both_layer,
                 "max": self.layer_1_max,
-                "abs_max": self.layer_1_max
+                "abs_max": self.layer_1_max,
             },
             self.layer_2_name: {
                 "mean": mean_2,
@@ -85,8 +77,8 @@ class GradientStatsTrackerTest(TestCase):
                 "min": self.layer_2_min,
                 "abs_min": self.absolute_min_both_layer,
                 "max": self.layer_2_max,
-                "abs_max": math.fabs(self.layer_2_min)  # since the absolute value is higher than the normal max
-            }
+                "abs_max": math.fabs(self.layer_2_min),  # since the absolute value is higher than the normal max
+            },
         }
 
         mean_1 = 0.195
@@ -102,7 +94,7 @@ class GradientStatsTrackerTest(TestCase):
                 "min": self.layer_1_min * 2,
                 "abs_min": self.absolute_min_both_layer,
                 "max": self.layer_1_max * 2,
-                "abs_max": self.layer_1_max * 2
+                "abs_max": self.layer_1_max * 2,
             },
             self.layer_2_name: {
                 "mean": mean_2,
@@ -111,8 +103,8 @@ class GradientStatsTrackerTest(TestCase):
                 "min": self.layer_2_min * 2,
                 "abs_min": self.absolute_min_both_layer,
                 "max": self.layer_2_max * 2,
-                "abs_max": math.fabs(self.layer_2_min * 2)  # since the absolute value is higher than the normal max
-            }
+                "abs_max": math.fabs(self.layer_2_min * 2),  # since the absolute value is higher than the normal max
+            },
         }
 
         mean_1 = 0.26
@@ -128,7 +120,7 @@ class GradientStatsTrackerTest(TestCase):
                 "min": self.layer_1_min * 3,
                 "abs_min": self.absolute_min_both_layer,
                 "max": self.layer_1_max * 3,
-                "abs_max": self.layer_1_max * 3
+                "abs_max": self.layer_1_max * 3,
             },
             self.layer_2_name: {
                 "mean": mean_2,
@@ -137,8 +129,8 @@ class GradientStatsTrackerTest(TestCase):
                 "min": self.layer_2_min * 3,
                 "abs_min": self.absolute_min_both_layer,
                 "max": self.layer_2_max * 3,
-                "abs_max": math.fabs(self.layer_2_min * 3)  # since the absolute value is higher than the normal max
-            }
+                "abs_max": math.fabs(self.layer_2_min * 3),  # since the absolute value is higher than the normal max
+            },
         }
 
     def test_compute_two_layers_statistic(self):
@@ -158,8 +150,9 @@ class GradientStatsTrackerTest(TestCase):
             layer_2_params = MagicMock()
             layer_2_params.grad = self.layer_2_gradients * batch_number
 
-            named_parameters = ((n, p)
-                                for n, p in [(self.layer_1_name, layer_1_params), (self.layer_2_name, layer_2_params)])
+            named_parameters = (
+                (n, p) for n, p in [(self.layer_1_name, layer_1_params), (self.layer_2_name, layer_2_params)]
+            )
 
             self.tracker.batch_statistic_upgrade(named_parameters=named_parameters)
 
@@ -177,6 +170,59 @@ class GradientStatsTrackerTest(TestCase):
             self.assertAlmostEqual(float(expected_value), float(actual_value), places=3)
 
 
+class GradientTrackerTest(TestCase):
+    def test_keep_good_layer(self):
+        # pylint: disable=protected-access
+        gradient_tracker = GradientTracker(keep_bias=False)
+        layer_to_keep_params = MagicMock()
+        layer_to_keep_params.requires_grad = True
+        self.assertTrue(gradient_tracker._keep_layer(layer_to_keep_params, "fake_layer_name_to_keep"))
+        self.assertFalse(gradient_tracker._keep_layer(layer_to_keep_params, "bias_name_not_to_keep"))
+
+        layer_not_to_keep_params = MagicMock()
+        layer_not_to_keep_params.requires_grad = False
+        self.assertFalse(gradient_tracker._keep_layer(layer_not_to_keep_params, "fake_layer_name_not_to_keep"))
+        self.assertFalse(gradient_tracker._keep_layer(layer_not_to_keep_params, "bias_name_not_to_keep"))
+
+        gradient_tracker = GradientTracker(keep_bias=True)
+        layer_to_keep_params = MagicMock()
+        layer_to_keep_params.requires_grad = True
+        self.assertTrue(gradient_tracker._keep_layer(layer_to_keep_params, "fake_layer_name_to_keep"))
+        self.assertTrue(gradient_tracker._keep_layer(layer_to_keep_params, "bias_name_to_keep"))
+
+        layer_not_to_keep_params = MagicMock()
+        layer_not_to_keep_params.requires_grad = False
+        self.assertFalse(gradient_tracker._keep_layer(layer_not_to_keep_params, "fake_layer_name_not_to_keep"))
+        self.assertFalse(gradient_tracker._keep_layer(layer_not_to_keep_params, "bias_name_not_to_keep"))
+
+    def test_integration(self):
+        train_gen = some_data_generator(20)
+        valid_gen = some_data_generator(20)
+        torch.manual_seed(42)
+        self.pytorch_network = nn.Linear(1, 1)
+        self.loss_function = nn.MSELoss()
+        self.optimizer = torch.optim.SGD(self.pytorch_network.parameters(), lr=1e-3)
+        device = torch.device("cpu")
+        self.model = Model(self.pytorch_network, self.optimizer, self.loss_function, device=device)
+        gradient_tracker = GradientTracker()
+
+        self.model.fit_generator(train_gen, valid_gen, epochs=10, steps_per_epoch=5, callbacks=[gradient_tracker])
+
+    @skipIf(not torch.cuda.is_available(), "no gpu available")
+    def test_integration_on_gpu(self):
+        train_gen = some_data_generator(20)
+        valid_gen = some_data_generator(20)
+        torch.manual_seed(42)
+        self.pytorch_network = nn.Linear(1, 1)
+        self.loss_function = nn.MSELoss()
+        self.optimizer = torch.optim.SGD(self.pytorch_network.parameters(), lr=1e-3)
+        device = torch.device("cuda:0")
+        self.model = Model(self.pytorch_network, self.optimizer, self.loss_function, device=device)
+        gradient_tracker = GradientTracker()
+
+        self.model.fit_generator(train_gen, valid_gen, epochs=10, steps_per_epoch=5, callbacks=[gradient_tracker])
+
+
 @skipIf(SummaryWriter is None, "Unable to import SummaryWriter from torch")
 class TensorBoardGradientTrackerTest(TestCase):
     batch_size = 20
@@ -266,25 +312,28 @@ class TensorBoardGradientTrackerTest(TestCase):
             if self.num_layer > 1:
                 layer_names = []
                 for layer_idx in range(self.num_layer):
-                    layer_names.append("{}.".format(layer_idx))
+                    layer_names.append(f"{layer_idx}.")
             for layer_name in layer_names:
-                expected_calls.append(call('gradient_distributions/{}weight'.format(layer_name), {'mean': ANY}, epoch))
+                expected_calls.append(call(f'gradient_distributions/{layer_name}weight', {'mean': ANY}, epoch))
                 expected_calls.append(
-                    call('gradient_distributions/{}weight'.format(layer_name), {'mean_std_dev_up': ANY}, epoch))
+                    call(f'gradient_distributions/{layer_name}weight', {'mean_std_dev_up': ANY}, epoch)
+                )
                 expected_calls.append(
-                    call('gradient_distributions/{}weight'.format(layer_name), {'mean_std_dev_down': ANY}, epoch))
-                expected_calls.append(call('other_gradient_stats/{}weight'.format(layer_name), {'min': ANY}, epoch))
-                expected_calls.append(call('other_gradient_stats/{}weight'.format(layer_name), {'max': ANY}, epoch))
+                    call(f'gradient_distributions/{layer_name}weight', {'mean_std_dev_down': ANY}, epoch)
+                )
+                expected_calls.append(call(f'other_gradient_stats/{layer_name}weight', {'min': ANY}, epoch))
+                expected_calls.append(call(f'other_gradient_stats/{layer_name}weight', {'max': ANY}, epoch))
 
                 if keep_bias:
-                    expected_calls.append(call('gradient_distributions/{}bias'.format(layer_name), {'mean': ANY},
-                                               epoch))
+                    expected_calls.append(call(f'gradient_distributions/{layer_name}bias', {'mean': ANY}, epoch))
                     expected_calls.append(
-                        call('gradient_distributions/{}bias'.format(layer_name), {'mean_std_dev_up': ANY}, epoch))
+                        call(f'gradient_distributions/{layer_name}bias', {'mean_std_dev_up': ANY}, epoch)
+                    )
                     expected_calls.append(
-                        call('gradient_distributions/{}bias'.format(layer_name), {'mean_std_dev_down': ANY}, epoch))
-                    expected_calls.append(call('other_gradient_stats/{}bias'.format(layer_name), {'min': ANY}, epoch))
-                    expected_calls.append(call('other_gradient_stats/{}bias'.format(layer_name), {'max': ANY}, epoch))
+                        call(f'gradient_distributions/{layer_name}bias', {'mean_std_dev_down': ANY}, epoch)
+                    )
+                    expected_calls.append(call(f'other_gradient_stats/{layer_name}bias', {'min': ANY}, epoch))
+                    expected_calls.append(call(f'other_gradient_stats/{layer_name}bias', {'max': ANY}, epoch))
 
         method_calls = self.writer.add_scalars.mock_calls
         self.assertEqual(len(method_calls), len(expected_calls))
